@@ -25,6 +25,7 @@ from .model.model_router import ModelRouter
 from .model_type import ModelType, determine_model_type
 from .reducer.combined_reducer import CombinedReducer
 from .selector.selector import Selector
+from .threshold_callback import ThresholdCallback
 from .weights.combined_weights import CombinedWeights
 from .weights.weights import WEIGHTS_COLUMN
 from .windower.windower import Windower
@@ -56,7 +57,6 @@ def _assign_bin(timestamp, bins: list[datetime.datetime]) -> int:
 def _best_trial(
     study: optuna.Study, dt: datetime.datetime | None = None
 ) -> optuna.trial.FrozenTrial:
-    """
     if dt is None:
         return study.best_trial
     min_trial = None
@@ -74,9 +74,6 @@ def _best_trial(
     if min_trial is None:
         min_trial = study.best_trial
     return min_trial
-    """
-    best_brier = min(study.best_trials, key=lambda t: t.values[1])
-    return best_brier
 
 
 class Trainer(Fit):
@@ -216,7 +213,7 @@ class Trainer(Fit):
             load_if_exists=True,
             sampler=restored_sampler,
             directions=[
-                optuna.study.StudyDirection.MAXIMIZE,
+                # optuna.study.StudyDirection.MAXIMIZE,
                 optuna.study.StudyDirection.MINIMIZE,
             ],
         )
@@ -258,8 +255,8 @@ class Trainer(Fit):
                 save: bool,
                 split_idx: datetime.datetime,
                 no_evaluation: bool,
-            ) -> tuple[float, float]:
-                # ) -> float:
+                # ) -> tuple[float, float]:
+            ) -> float:
                 print(f"Beginning trial for: {split_idx.isoformat()}")
                 trial.set_user_attr(_IDX_USR_ATTR_KEY, split_idx.isoformat())
                 folder = os.path.join(
@@ -298,7 +295,8 @@ class Trainer(Fit):
                             os.removedirs(folder)
                         logging.warning("Y train only contains 1 unique datapoint.")
                         # return _BAD_OUTPUT, -_BAD_OUTPUT
-                        return _BAD_OUTPUT, -_BAD_OUTPUT
+                        # return _BAD_OUTPUT, -_BAD_OUTPUT
+                        return -_BAD_OUTPUT
                     print(f"Windowing took {time.time() - start_windower}")
 
                     # Perform common reductions
@@ -364,13 +362,13 @@ class Trainer(Fit):
                     print(f"Calibrating took {time.time() - start_calibrate}")
 
                     # Output
-                    cal_pred = calibrator.transform(
+                    y_pred = calibrator.transform(
                         y_pred if calibrator.predictions_as_x(y_test) else x_test
                     )
-                    cal_pred[PREDICTION_COLUMN] = y_pred[PREDICTION_COLUMN]
                     output = 0.0
                     loss = 0.0
                     pvalue = 0.0
+                    log_loss_value = 0.0
                     if determine_model_type(y_series) == ModelType.REGRESSION:
                         output = float(r2_score(y_test, y_pred[[PREDICTION_COLUMN]]))
                         print(f"R2: {output}")
@@ -380,11 +378,12 @@ class Trainer(Fit):
                         prob_col = PROBABILITY_COLUMN_PREFIX + str(1)
                         if prob_col in y_pred.columns.values.tolist():
                             loss = float(brier_score_loss(y_test, y_pred[[prob_col]]))
+                            log_loss_value = float(
+                                log_loss(y_test.astype(float), y_pred[[prob_col]])
+                            )
                             pvalue = calibrator.p_value
                             print(f"Brier: {loss}")
-                            print(
-                                f"Log Loss: {float(log_loss(y_test.astype(float), y_pred[[prob_col]]))}"
-                            )
+                            print(f"Log Loss: {log_loss_value}")
                             print(f"P-Value: {pvalue}")
                         print(
                             f"Accuracy: {float(accuracy_score(y_test, y_pred[[PREDICTION_COLUMN]]))}"
@@ -413,14 +412,16 @@ class Trainer(Fit):
                             )
 
                     # return output, loss
-                    return pvalue, loss
+                    # return pvalue, loss
+                    return loss
                 except WavetrainException as exc:
                     print(str(exc))
                     logging.warning(str(exc))
                     if new_folder:
                         os.removedirs(folder)
                     # return _BAD_OUTPUT, -_BAD_OUTPUT
-                    return _BAD_OUTPUT, -_BAD_OUTPUT
+                    # return _BAD_OUTPUT, -_BAD_OUTPUT
+                    return -_BAD_OUTPUT
 
             start_validation_index = (
                 dt_index.to_list()[-int(len(dt_index) * self._validation_size) - 1]
@@ -441,8 +442,8 @@ class Trainer(Fit):
                 ].to_list()[0]
             )
 
-            def test_objective(trial: optuna.Trial) -> tuple[float, float]:
-                # def test_objective(trial: optuna.Trial) -> float:
+            # def test_objective(trial: optuna.Trial) -> tuple[float, float]:
+            def test_objective(trial: optuna.Trial) -> float:
                 return _fit(
                     trial,
                     test_df,
@@ -527,18 +528,25 @@ class Trainer(Fit):
                         trial: optuna.Trial,
                         idx: datetime.datetime,
                         series: pd.Series,
-                    ) -> tuple[float, float]:
-                        # ) -> float:
+                        # ) -> tuple[float, float]:
+                    ) -> float:
                         return _fit(trial, test_df.copy(), series, False, idx, False)
 
                     study.optimize(
                         functools.partial(
                             validate_objective, idx=test_idx, series=test_series
                         ),
-                        n_trials=1,
+                        n_trials=self._trials,
                         timeout=None
                         if self._max_train_timeout is None
                         else self._max_train_timeout.total_seconds(),
+                        callbacks=[
+                            ThresholdCallback(
+                                0.0
+                                if study.best_trial.value is None
+                                else study.best_trial.value
+                            )
+                        ],
                     )
                 else:
                     break
@@ -549,7 +557,7 @@ class Trainer(Fit):
                     test_series,
                     True,
                     test_idx,
-                    True,
+                    False,
                 )
                 print(f"Saved model with value: {value}")
                 last_processed_dt = test_idx
